#ifndef POINTER_GESTURES_SERVER_PROTOCOL_H
#define POINTER_GESTURES_SERVER_PROTOCOL_H

#ifdef  __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include <stddef.h>
#include "wayland-server.h"

struct wl_client;
struct wl_resource;

struct _wl_pointer_gesture_pinch;
struct _wl_pointer_gesture_swipe;
struct _wl_pointer_gestures;
struct wl_pointer;
struct wl_surface;

extern const struct wl_interface _wl_pointer_gestures_interface;
extern const struct wl_interface _wl_pointer_gesture_swipe_interface;
extern const struct wl_interface _wl_pointer_gesture_pinch_interface;

#ifndef _WL_POINTER_GESTURES_ERROR_ENUM
#define _WL_POINTER_GESTURES_ERROR_ENUM
enum _wl_pointer_gestures_error {
	_WL_POINTER_GESTURES_ERROR_VERSION_MISMATCH = 0,
};
#endif /* _WL_POINTER_GESTURES_ERROR_ENUM */

/**
 * _wl_pointer_gestures - touchpad gestures
 * @get_swipe_gesture: get swipe gesture
 * @get_pinch_gesture: get pinch gesture
 *
 * A global interface to provide semantic touchpad gestures for a given
 * pointer.
 *
 * Two gestures are currently supported: swipe and zoom/rotate. All
 * gestures follow a three-stage cycle: begin, update, end and are
 * identified by a unique id.
 *
 * Warning! The protocol described in this file is experimental. Each
 * version of this protocol should be considered incompatible with any
 * other version, and a client binding to a version different to the one
 * advertised will be terminated. Once the protocol is declared stable,
 * compatibility is guaranteed, the '_' prefix will be removed from the
 * name and the version will be reset to 1.
 */
struct _wl_pointer_gestures_interface {
	/**
	 * get_swipe_gesture - get swipe gesture
	 * @id: (none)
	 * @pointer: (none)
	 *
	 * Create a swipe gesture object. See the
	 * wl_pointer_gesture_swipe interface for details.
	 */
	void (*get_swipe_gesture)(struct wl_client *client,
				  struct wl_resource *resource,
				  uint32_t id,
				  struct wl_resource *pointer);
	/**
	 * get_pinch_gesture - get pinch gesture
	 * @id: (none)
	 * @pointer: (none)
	 *
	 * Create a pinch gesture object. See the
	 * wl_pointer_gesture_pinch interface for details.
	 */
	void (*get_pinch_gesture)(struct wl_client *client,
				  struct wl_resource *resource,
				  uint32_t id,
				  struct wl_resource *pointer);
};


/**
 * _wl_pointer_gesture_swipe - a swipe gesture object
 * @destroy: destroy the pointer swipe gesture object
 *
 * A swipe gesture object notifies a client about a multi-finger swipe
 * gesture detected on an indirect input device such as a touchpad. The
 * gesture is usually initiated by multiple fingers moving in the same
 * direction but once initiated the direction may change. The precise
 * conditions of when such a gesture is detected are
 * implementation-dependent.
 *
 * A gesture consists of three stages: begin, update (optional) and end.
 * There cannot be multiple simultaneous pinch or swipe gestures on a same
 * pointer/seat, how compositors prevent these situations is
 * implementation-dependent.
 *
 * A gesture may be cancelled by the compositor or the hardware. Clients
 * should not consider performing permanent or irreversible actions until
 * the end of a gesture has been received.
 */
struct _wl_pointer_gesture_swipe_interface {
	/**
	 * destroy - destroy the pointer swipe gesture object
	 *
	 * 
	 */
	void (*destroy)(struct wl_client *client,
			struct wl_resource *resource);
};

#define _WL_POINTER_GESTURE_SWIPE_BEGIN	0
#define _WL_POINTER_GESTURE_SWIPE_UPDATE	1
#define _WL_POINTER_GESTURE_SWIPE_END	2

#define _WL_POINTER_GESTURE_SWIPE_BEGIN_SINCE_VERSION	1
#define _WL_POINTER_GESTURE_SWIPE_UPDATE_SINCE_VERSION	1
#define _WL_POINTER_GESTURE_SWIPE_END_SINCE_VERSION	1

static inline void
_wl_pointer_gesture_swipe_send_begin(struct wl_resource *resource_, uint32_t serial, uint32_t time, struct wl_resource *surface, uint32_t fingers)
{
	wl_resource_post_event(resource_, _WL_POINTER_GESTURE_SWIPE_BEGIN, serial, time, surface, fingers);
}

static inline void
_wl_pointer_gesture_swipe_send_update(struct wl_resource *resource_, uint32_t time, wl_fixed_t dx, wl_fixed_t dy)
{
	wl_resource_post_event(resource_, _WL_POINTER_GESTURE_SWIPE_UPDATE, time, dx, dy);
}

static inline void
_wl_pointer_gesture_swipe_send_end(struct wl_resource *resource_, uint32_t serial, uint32_t time, int32_t cancelled)
{
	wl_resource_post_event(resource_, _WL_POINTER_GESTURE_SWIPE_END, serial, time, cancelled);
}

/**
 * _wl_pointer_gesture_pinch - a pinch gesture object
 * @destroy: destroy the pinch gesture object
 *
 * A pinch gesture object notifies a client about a multi-finger pinch
 * gesture detected on an indirect input device such as a touchpad. The
 * gesture is usually initiated by multiple fingers moving towards each
 * other or away from each other, or by two or more fingers rotating around
 * a logical center of gravity. The precise conditions of when such a
 * gesture is detected are implementation-dependent.
 *
 * A gesture consists of three stages: begin, update (optional) and end.
 * There cannot be multiple simultaneous pinch or swipe gestures on a same
 * pointer/seat, how compositors prevent these situations is
 * implementation-dependent.
 *
 * A gesture may be cancelled by the compositor or the hardware. Clients
 * should not consider performing permanent or irreversible actions until
 * the end of a gesture has been received.
 */
struct _wl_pointer_gesture_pinch_interface {
	/**
	 * destroy - destroy the pinch gesture object
	 *
	 * 
	 */
	void (*destroy)(struct wl_client *client,
			struct wl_resource *resource);
};

#define _WL_POINTER_GESTURE_PINCH_BEGIN	0
#define _WL_POINTER_GESTURE_PINCH_UPDATE	1
#define _WL_POINTER_GESTURE_PINCH_END	2

#define _WL_POINTER_GESTURE_PINCH_BEGIN_SINCE_VERSION	1
#define _WL_POINTER_GESTURE_PINCH_UPDATE_SINCE_VERSION	1
#define _WL_POINTER_GESTURE_PINCH_END_SINCE_VERSION	1

static inline void
_wl_pointer_gesture_pinch_send_begin(struct wl_resource *resource_, uint32_t serial, uint32_t time, struct wl_resource *surface, uint32_t fingers)
{
	wl_resource_post_event(resource_, _WL_POINTER_GESTURE_PINCH_BEGIN, serial, time, surface, fingers);
}

static inline void
_wl_pointer_gesture_pinch_send_update(struct wl_resource *resource_, uint32_t time, wl_fixed_t dx, wl_fixed_t dy, wl_fixed_t scale, wl_fixed_t rotation)
{
	wl_resource_post_event(resource_, _WL_POINTER_GESTURE_PINCH_UPDATE, time, dx, dy, scale, rotation);
}

static inline void
_wl_pointer_gesture_pinch_send_end(struct wl_resource *resource_, uint32_t serial, uint32_t time, int32_t cancelled)
{
	wl_resource_post_event(resource_, _WL_POINTER_GESTURE_PINCH_END, serial, time, cancelled);
}

#ifdef  __cplusplus
}
#endif

#endif
